/** 
 *  @file   IRingIO.xdc
 *
 *  @brief      TODO
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


package ti.syslink.ipc.rtos.interfaces;

import ti.syslink.ipc.rtos.interfaces.SysLink ;
import xdc.runtime.Error;
import xdc.runtime.IGateProvider;
import xdc.runtime.IHeap;
import ti.sdo.ipc.GateMP;
import ti.sdo.ipc.SharedRegion;
import ti.sdo.utils.NameServer;
import ti.sdo.utils.MultiProc;
import ti.syslink.ipc.rtos.interfaces.IClientNotifyMgr ;

interface IRingIO {

   /*! @_nodoc
     *  Delegate writer needs...
     */
    const UInt32 CREATED = 0x05251995;
    const UInt32 VERSION = 1;
   /*! Type of creation/open */
    const UInt32 STATIC_CREATE  = 1;
    /*! Type of creation/open */
    const UInt32 DYNAMIC_CREATE = 2;
    /*! Type of creation/open */
    const UInt32 STATIC_OPEN    = 4;
    /*! Type of creation/open */
    const UInt32 DYNAMIC_OPEN   = 8;


    /*!
     * =============invalidOffset===============
     *  This constant an invalid index in the ring buffer.
     */
    const UInt32 invalidOffset = -1;

    /*!
     * ============invalidAttr================
     *
     *  This constant denotes an invalid attribute type.
     *  This should not be used for any valid attribute.
     */
    const UInt16 invalidAttr = 0xFFFF;

    /*!
     * ============maxNameLen================
     *  Maximum length of the  name strings .
     */
    const UInt32  maxNameLen = 32;

    /*!
     *  Maximum number of instances managed by RingIO module.
     */
    const UInt32 maxRuntimeEntries = 32;

    /*!
     *  Maximum number of instances managed by RingIO module.
     */
    const UInt32 notifyEventNo = 3;

    /*!
     *  If set  to true all RingIO instances will use dafault gate set in
     *  RingIO.
     *  If default gate is not set in RingIO, it uses GateMP's default
     *  gate. Set it to zero if RingIO needs to  create  gate for a
     *  instance (remoteProtect and localProtect  flags should be valid in
     *  params in this case).
     */
    config Bool useDefaultGate   = false;

    metaonly config NameServer.Params nameSrvPrms;

    /*!
     *  IGateProvider Proxy
     *
     *  Module used to provide default critical region management when accessing
     *  the shared memory. The default is {@link GatePeterson}.
     *  TODO: what about the parameters!
     *  TODO: nodoc?
     */
    proxy GateProxy inherits IGateProvider;


    /*!
     * Cache Size
     */
    const UInt32 cacheSize    = 128;

    /*!
     *  ============Flags=================================================
     *
     *  These constants denote the flags provided while opening the RingIO.
     */
     const Int32 DATABUF_MAINTAINCACHE = 0x1;
     const Int32 NEED_EXACT_SIZE  = 0x2;


    /*!
     *  ============NotifyType=================================================
     *
     *  Enumeration of RingIO notification types.
     *
     *  @value(NOTIFICATION_NONE)
     *              No notification required.
     *  @value(NOTIFICATION_ALWAYS)
     *              Notify whenever the other side sends data/frees up space.
     *  @value(NOTIFICATION_ONCE)
     *              Notify when the other side sends data/frees up space.
     *              Once the notification is done, the notification is disabled
     *              until it is enabled again
     *  @value(NOTIFICATION_HDWRFIFO_ALWAYS)
     *              Notify whenever the other side sends data/frees up space.
     *              This notification is never disabled.
     *  @value(NOTIFICATION_HDWRFIFO_ONCE)
     *              Notify when the other side sends data/frees up space.
     *              Once the notification is done, the notification is disabled
     *              until it is enabled again. The notification is enabled once
     *              the watermark is crossed and does not require buffer to get
     *              full/empty.
     *  ========================================================================
     */
    enum NotifyType {
        NOTIFICATION_NONE = 0,
        NOTIFICATION_ALWAYS,
        NOTIFICATION_ONCE,
        NOTIFICATION_HDWRFIFO_ALWAYS,
        NOTIFICATION_HDWRFIFO_ONCE
    };

    /*!
     *  Enumeration of RingIO open modes.
     *
     *  @value(MODE_READER)  Reader mode.
     *  @value(MODE_WRITER) Writer mode.
     *  ========================================================================
     */
    enum OpenMode {
        MODE_READER = 0,
        MODE_WRITER,
    };

    /*! This type is used for the Buffer Pointer Type. */
    typedef Ptr                                     BufPtr;

    /*!  This type is used for the Notification parameter */
    typedef Ptr                                     NotifyParam;

    /*!  This type is used for the Notification  Message.  */
    typedef UInt16                                  NotifyMsg;

    /*!
     *  ========NotifyFxn =====================================================
     *  Signature of function used as the Notification function.
     *
     *  @param(handle) Handle to the RingIO.
     *  @param(param)  Parameter used while registering the notification.
     *  @param(msg)    Message passed along with notification.
     *
     *   @b(returns)    None.
     *  ========================================================================
     */
    typedef void (* NotifyFxn ) (Handle, Ptr, NotifyMsg);

     /*!
     * ============ CLIENT_PADDING ================
     * Padding required for the client control structure.
     * 2  * sizeof (RingIO_BufPtr)
     * 11 * sizeof (UInt32)
     * 1  * sizeof (Bool)
     * 2  * sizeof (UInt16)
     */
     config UInt32  CLIENT_PADDING = (128 - ((4 * 14) + (1 * 1) /2)) ;

     /*!
     * ============ CONTROLSTRUCT_PADDING================
     * Padding required for the client control structure.
     *
     */
     config UInt32  CONTROLSTRUCT_PADDING = (128 - ((4 * 14) + (2 * 2) /2)) ;

     /*!
      * ============= RingIO_Config ============
      * Structure defining config parameters for the RingIO module.
      *  @param(maxRuntimeEntries)  Maximum number of RingIOs that can be dynamically created
      *  @param(maxNameLen)  Maximum length for RingIO names
      */
     struct Config {
         UInt32        maxRuntimeEntries;
         UInt32        maxNameLen;
         UInt32        notifyEventNo;
     };

     /*!
     *   Notify event number to be used by the Client Notify Manager
     * ============ Attr ================
     * @param(offset) Private structure of RingIO Attribute buffer
     * @param(prevAttrOffset) Offset into data buffer
     * @param(type) Offset to previous attribute
     * @param(size) Attribute type
     * @param(param) Size of the attribute payload following this
      */
     struct Attr {
         UInt32    offset;
         Int32     prevoffset;
         UInt16    type;
         UInt16    size;
         UInt32    param;
     };

    /*!
     *  =========Client================================================
     *
     *  This structure defines the RingIO Shared memory client structure.
     *  The RingIO Reader or Writer state information is stored in this
     *  structure
     *  @value(pDataStart)
     *              Virtual start address of the data buffer
     *  @value(pAttrStart)
     *              Virtual start address of the attr buffer
     *  @value(acqStart)
     *              Start offset of data buffer that has been acquired by the
     *              application.
     *  @value(acqSize)
     *              Size of data that has been acquired
     *  @value(acqAttrStart)
     *              Start offset of the acquired attribute buffer
     *  @value(acqAttrSize)
     *              Size of attribute data that has been acquired
     *  @value(notifyType)
     *              Notification type
     *  @value(notifyParam)
     *              Parameter to the Notification function
     *  @value(notifyWaterMark)
     *              Watermark that should be satisfied before notification is
     *              done.
     *  @value(flags)
     *              Counting bits from MSB to LSB 0....31
     *              B31 - Denotes whether the client uses cache for accessing
     *                    the data buffer (dataBufCacheUseFlag)
     *              B30 - Denotes whether requests for acquire buffer should
     *                    always be satisfied. If a contiguous request could not
     *                    be satisfied because of the buffer wrap, we need to
     *                    mark an early end and provide only a contiguous buffer
     *                    (contiguousBufFlag)
     *  @value(isValid)
     *              Indicates whether the Client is initialized
     *  @value(refCount)
     *              Denotes whether ringio has been re-opened or not
     *  @value(ClientNotifyMgr_Handle)
     *              Denotes ClientNotifyMgr Handle
     *  @value(procId)
     *              Processor Id where the client is executing
     *  @value(isRegistered)
     *              If TRUE, denotes client has registered notification function
     *  @value(padding)
     *              Padding.
     *  @value(clientNotifyMgrHandle)
     *              Handle to the clientNotifyMgr handle used for this instance
     *  @value(clientNotifyMgrGate)
     *              Handle to the gate to the clientNotifyMgr handle used for
     *              this instance. This is used only for opening
     *  ========================================================================
     */
     struct ClientStruct {
        BufPtr          pDataStart;
        BufPtr          pAttrStart;
        UInt32          acqStart;
        UInt32          acqSize;
        UInt32          acqAttrStart;
        UInt32          acqAttrSize;
        UInt32          notifyType;
        NotifyParam     notifyParam;
        UInt32          notifyWaterMark;
        UInt32          flags;
        UInt32          isValid;
        UInt32          refCount;
        UInt32          notifyId;
        Bool            isRegistered;
        UInt16          padding [CLIENT_PADDING];
        IClientNotifyMgr.Handle clientNotifyMgrHandle;
        GateMP.Handle           clientNotifyMgrGate;
    };

    /*!
     *  ========ControlStruct==================================================
     *
     *  This structure defines the RingIO Control Structure. This structure
     *  is stored in shared memory and is accessible by all clients. The
     *  control structure supports a single reader and a single writer for
     *  the ring buffer.
     *
     *  @value(srPtrBufStart)
     *              Physical start address of the data buffer.
     *  @value(srPtrBufEnd)
     *              Total size of the Data buffer (offset from srPtrBufStart)
     *  @value(curBufEnd)
     *              Current buffer size. This may be <= dataBufEnd (offset from
     *              srPtrBufStart)
     *  @value(dataBufEnd)
     *              Size of the main data buffer (offset from srPtrBufStart)
     *  @value(dataSharedAddrSize)
     *              Size of the data buffer
     *  @value(footBufSize)
     *              Size of the foot buffer used for providing linear buffer to
     *              a reader
     *  @value(validSize)
     *              Amount of valid data available in the data buffer. Valid
     *              Data is the total data that is readable by the reader using
     *              an acquire call. This  does not include the size of the data
     *              buffer already acquired by the reader
     *  @value(emptySize)
     *              Amount of empty space in the data buffer. This does not
     *              include the empty space already acquired by the writer
     *  @value(srPtrAttrBufStart)
     *              Physical start address of the attr buffer
     *  @value(srPtrAttrBufEnd)
     *              Total Size of the attribute buffer (offset)
     *  @value(curAttrBufEnd)
     *              Current Attr buffer size. This may be <= the srPtrAttrBufEnd
     *              (offset)
     *  @value(validAttrSize)
     *              Amount of valid attribute bytes available in the attribute
     *              buffer. The valid attribute bytes does not include the
     *              attribute bytes already acquired by the reader
     *  @value(emptyAttrSize)
     *              Amount of empty space in the attr buffer. This does not
     *              include the empty attr space already acquired by the writer
     *  @value(prevAttrOffset)
     *              Offset of the most recent attribute
     *  @value(procId)
     *              Processor Id where the client is executing
     *  @value(padding)
     *              Padding.
     *  ========================================================================
     */
    struct ControlStruct {
        UInt32                srPtrBufStart;
        UInt32                srPtrBufEnd;
        UInt32                curBufEnd;
        UInt32                dataBufEnd;
        UInt32                dataSharedAddrSize;
        UInt32                footBufSize;
        UInt32                validSize;
        UInt32                emptySize;
        UInt32                srPtrAttrBufStart;
        UInt32                srPtrAttrBufEnd;
        UInt32                curAttrBufEnd;
        UInt32                validAttrSize;
        UInt32                emptyAttrSize;
        Int32                 prevAttrOffset;
        UInt16                procId;
        UInt16                padding [CONTROLSTRUCT_PADDING];
    };


    /*!
     *  @brief  Structure defining config parameters for opening the
     *          RingIO instances.
     * @value(flags)
     *   Cache flags. See RingIO_Flags
     * @value(openMode)
     *   Mode of the client in open call.
     */
     struct openParams {
        UInt32      flags;
        UInt32      openMode;
    };


     /*!
      *  ======== getConfig ========
      * Function to initialize the Config parameter structure with default values.
      *
      *  @param(cfgParams) Structure of type RingIO_Config
      *
      *  @b(returns)       None.
      *
      */
     Void getConfig (Config * cfgParams);

//     /*!
//      *  ======== setup ========
//      * Function to setup the RingIO module.
//      *
//      *  @param(cfgParams) Structure of type RingIO_Config
//      *
//      *  @b(returns)       Success if status >= 0.
//      *                    Failure if status < 0.
//      *
//      */
//     Int32 setup (const Config * cfgParams);
//
//     /*!
//      *  ======== Destroy ========
//      *  Finalize the RingIO
//      */
//     Int32 destroy ();

instance:

     /*!
      *  name
      *     Name of the instance
      */
     config String      name = null;
     /*!
      *  The name must be unique among all RingIO
      *  instances in the entire system.  When creating a new
      *  RingIO, it is necessary to supply an instance name.
      */
     config UInt16     ctrlRegionId = SharedRegion.INVALIDREGIONID;
     /*! Shared region ID for RingIO control structures
      *
      *  The index corresponding to the shared region from which shared memory
      *  will be allocated if ctrlSharedAddr is NULL.
      *  If a valid ctrlSharedAddr is provided, this field is ignored.
      */
     config UInt16     dataRegionId = SharedRegion.INVALIDREGIONID;
     /*! Shared region ID for RingIO data buffer
      *
      *  The index corresponding to the shared region from which shared memory
      *  will be allocated if dataSharedAddr is NULL.
      *  If a valid dataSharedAddr is provided, this field is ignored.
      */
     config UInt16     attrRegionId = SharedRegion.INVALIDREGIONID;
     /*! Shared region ID for RingIO attr buffer
      *
      *  The index corresponding to the shared region from which shared memory
      *  will be allocated if attrSharedAddr is NULL.
      *  If a valid attrSharedAddr is provided, this field is ignored.
      */
     /*!
      *  ctrlSharedAddr
      *     Shared region address for control structure
      */
     config Ptr         ctrlSharedAddr = null;

     /*!
      * dataSharedAddr
      *     Shared Region Address for data buffer
      */
     config Ptr         dataSharedAddr = null;

     /*!
      * dataSharedAddrSize
      *     Data Buffer size
      */
     config UInt32      dataSharedAddrSize = 0;

     /*!
      * footBufSize
      *     Foot Buffer size
      */
     config UInt32      footBufSize = 0;

     /*!
      * attrSharedAddr
      *     Shared Region Address for attr buffer
      */
     config Ptr         attrSharedAddr = null;

     /*!
      * attrSharedAddrSize
      *     Attr Buffer size
      */
     config UInt32      attrSharedAddrSize = 0;

     /*!
      * flags
      *     Cache flags. See RingIO_Flags
      */
     config UInt32      flags = 0;

     /*!
      * remoteProcId
      *     Remote processor id with which the RingIO communicates
      */
     config UInt16        remoteProcId = 0;

     /*!
      * gateHandle
      *     Gate for protection
      */
     config GateMP.Handle gateHandle = null;

     /*! Local protection level for the module instance.*/
     config GateMP.LocalProtect  localProtect = GateMP.LocalProtect_INTERRUPT;

     /*! Remote protection level for the module instance.*/
     config GateMP.RemoteProtect remoteProtect = GateMP.RemoteProtect_SYSTEM;

     /*!
      *  openFlag
      *     Denotes whether create or open is to be called
      */
     config Bool          openFlag      = true;

     /*!
      *  Config
      *     Denotes  configuration to be passed to the transport
      */
     config Config        cfg = { maxRuntimeEntries:maxRuntimeEntries,
                                  maxNameLen:maxNameLen,
                                  notifyEventNo:notifyEventNo};


     config UInt32        openMode     = -1;

     /*!
      *  Maximum number of instances managed by RingIO module.
      */
     config       Ptr    transport     = null;

     /*!
      *  ======== setNotifyType =============================================
      *  Set notification type of client
      *
      *  @b(returns)    <status>
      *  ========================================================================
      */
     Int32 setNotifyType (NotifyType  notifyType);


    /*!
     *  ======== getValidSize ==================================================
     *  Returns the current valid data size
     *
     *  @b(returns)    <current valid data size>
     *  ========================================================================
     */
    UInt32
    getValidSize ();

    /*!
     *  ======== getEmptySize ==================================================
     *  Returns the current empty buffer size
     *
     *  @b(returns)    <current empty buffer size>
     *  ========================================================================
     */
    UInt32
    getEmptySize ();

    /*!
     *  ======== getValidAttrSize===============================================
     *  Returns the current valid attribute size
     *
     *  @b(returns)    <current valid attribute size>
     *  ========================================================================
     */
    UInt32
    getValidAttrSize ();

    /*!
     *  ======== getEmptyAttrSize===============================================
     *  Returns the current empty attribute buffer size
     *
     *  @b(returns)    <current empty attribute buffer size>
     *  ========================================================================
     */
    UInt32
    getEmptyAttrSize ();

    /*!
     *  ======== getAcquiredOffset =============================================
     *  Returns the current acquire offset for the client
     *
     *  @b(returns)    <client's current acquired offset>
     *  ========================================================================
     */
    UInt32 getAcquiredOffset ();

    /*!
     *  ======== getAcquiredSize =============================================
     *  Returns the size of buffer currently acquired
     *
     *  @b(returns)    <current acquired buffer size>
     *  ========================================================================
     */
    UInt32 getAcquiredSize ();

    /*!
     *  ======== getWatermark =============================================
     *  Returns the current watermark level specified by the client
     *
     *  @b(returns)    <current watermark level set by the client>
     *  ========================================================================
     */
    UInt32 getWaterMark ();

    /*!
     *  ======== setWatermark =============================================
     *  Sets the current watermark level of the client
     *
     *  @b(returns)    <Status>
     *  ========================================================================
     */
    Int32 setWaterMark (UInt32 watermark);

    /*!
     *  ======== acquire =============================================
     *  This function acquire a data buffer from RingIO.
     *  This function is used to open an RingIO Channel either for reading
     *  or writing. Only one reader and one writer can be opened on a RingIO
     *  Channel.
     *
     *  @param(pData) Location to store the pointer to the acquired data buffer.
     *  @param(pSize) Pointer to the size of data buffer acquired. Holds the
     *                size of buffer to be acquired. Returns the size of buffer
     *                actually acquired.
     *
     *  @b(returns)   Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    acquire (BufPtr * pData,
             UInt32 * pSize);

    /*!
     *  ======== release =============================================
     *  This function releases a data buffer to RingIO.
     *  This function releases an acquired buffer or part of it.
     *
     *  @param(size)   Size of data buffer to be released.
     *
     *  @b(returns)    Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    release (UInt32 size);

    /*!
     *  ======== cancel =============================================
     *  Cancel the previous acquire.
     *
     *  @b(returns)    Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    cancel ();

    /*!
     *  ======== getvAttribute =============================================
     *  This function gets an attribute with a variable-sized payload from
     *  the attribute buffer.
     *  If an attribute is present, the attribute type, the optional
     *  parameter, a pointer to the optional payload and the payload size
     *  are returned.
     *
     *  @param(type)  Location to receive the user-defined type of attribute.
     *  @param(param) Location to receive an optional parameter which depends on the
     *                attribute type.
     *  @param(vptr)  Pointer to buffer to receive the optional payload.
     *  @param(pSize) Location with the size of the variable attribute. On return,
     *                this stores the actual size of the payload.
     *
     *  @b(returns)   Status of the call (refer to above description).
     *  ============================================================================
     */
    Int32
    getvAttribute ( UInt16 *      type,
                    UInt32 *      param,
                    BufPtr        vptr,
                    UInt32 *      pSize);

    /*!
     *  ======== setvAttribute =============================================
     *  This function sets an attribute with a variable sized payload at the
     *  offset provided in the acquired data buffer.
     *  If the offset is not in the range of the acquired data buffer, the
     *  attribute is not set, and an error is returned. One exception to
     *  this rule is when no data buffer has been acquired. In this case an
     *  attribute is set at the next data buffer offset that can be acquired
     *
     *  @param(type)   User-defined type of attribute.
     *  @param(param)  Optional parameter which depends on the attribute type.
     *  @param(pdata)  Pointer to attribute payload buffer.
     *  @param(size)   Size of the attribute payload.
     *  @param(sendNotification)  Optional parameter whether to send
     *  Notification
     *
     *  @b(returns)    Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    setvAttribute (UInt16        type,
                   UInt32        param,
                   BufPtr        pdata,
                   UInt32        size,
                   Bool          sendNotification);


    /*!
     *  ======= getAttribute ===================================================
     *  This function gets a fixed-size attribute from the attribute buffer.
     *  If an attribute is present, the attribute type and a related
     *  parameter are returned.
     *
     *  @param(handle
     *              Handle to the RingIO Client.
     *  @param(type
     *              Location to receive the user-defined type of attribute.
     *  @param(param
     *              Location to receive an optional parameter which depends on
     *              the attribute type.
     *
     *  @b(returns)    Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    getAttribute (UInt16 *      type,
                  UInt32 *      param);


    /*!
     *  ======== setAttribute ==================================================
     *  This function sets a fixed-size attribute at the offset provided in
     *  the acquired data buffer.
     *  If the offset is not in the range of the acquired data buffer, the
     *  attribute is not set, and an error is returned. One exception to
     *  this rule is when no data buffer has been acquired. In this case an
     *  attribute is set at the next data buffer offset that can be acquired
     *
     *  @param(handle) Handle to the RingIO Client.
     *  @param(type)   User-defined type of attribute.
     *  @param(param)  Optional parameter which depends on the attribute type.
     *  @param(sendNotification)  Optional parameter whether to send
     *  Notification
     *
     *  @b(returns)    Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    setAttribute (UInt16        type,
                  UInt32        param,
                  Bool          sendNotification);

    /*!
     *  ======== flush ===================================================
     *  This function flushes the data buffer.
     *  This function is used to flush the data from the RingIO.
     *  Behavior of this function depends on the value of hardFlush paramument
     *  When hardFlush is false:
     *  If function is called for the writer, all the valid data in buffer
     *  after the first attribute location will be discarded. In case there
     *  are no attributes, no data will be cleared from the buffer. Note
     *  that this does not include the data that has been already acquired
     *  by the reader. Note that the attribute will also be cleared from the
     *  attribute buffer.
     *  For the reader, all the data till the next attribute location will
     *  be discarded. And if there is no attribute in the buffer, all valid
     *  data will get discarded. Note that the attribute will remain the
     *  attribute buffer. This is different from the behavior mentioned for
     *  the writer.
     *  When hardFlush is true:
     *  If function is called from the writer, all committed data and
     *  attributes that is not acquired by reader are removed from
     *  the RingIO instance. The writer pointer is moved to point to
     *  reader's head pointer
     *  If function is called from the reader, all data and attributes
     *  that can be subsequently acquired from the reader are removed.
     *
     *  @param(handle)    Handle to the RingIO Client.
     *  @param(hardFlush) Mode in which the flush operation discards committed
     *                    data and attributes
     *  @param(type)      Location to receive the User-defined type of
     *                    attribute.
     *  @param(param)     Location to receive an optional parameter which
     *                    depends on the attribute type.
     *  @param(bytesFlushed) Location to receive the size of the buffer flushed.
     *
     *  @b(returns) Status of the call (refer to above description).
     *
     *  ========================================================================
     */
    Int32
    flush (Bool          hardFlush,
           UInt16 *      type,
           UInt32 *      param,
           UInt32 *      bytesFlushed);

    /*!
     *  ======== registerNotifier ==============================================
     *  This function sets Notification parameters for the RingIO Client.
     *  Both the reader and writer can set their notification mechanism
     *  using this function
     *
     *  @param(notifyType)      Type of notification.
     *  @param(notifyWatermark) Watermark for notification
     *  @param(notifyFunc)      Function to call when notification is required
     *  @param(pNotifyParam)    Pointer to the notification parameter.
     *                          The type of the pointer and its size depends
     *                          on the notification function
     *
     *  @b(returns) Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    registerNotifier (NotifyType  notifyType,
                      UInt32      notifyWatermark,
                      NotifyFxn   notifyFunc,
                      Ptr         cbContext);

    /*!
     *  ======== unregisterNotifier ============================================
     *  This function unregisters Notification parameters for the RingIO Client.
     *  Both the reader and writer can unregister their notification mechanism
     *  using this function
     *
     *  @b(returns) Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    unregisterNotifier ();

    /*!
     *  ======= close ==========================================================
     *  This function closes a RingIO instance.
     *
     *  @b(returns)    Status of the call (refer to above description).
     *  ========================================================================
     */
    Int32
    close ();

    /*
     *  Send a notification to the other client manually
     */
    Int32
    sendNotify (NotifyMsg msg);

}
