/** 
 *  @file   IRingIOTransportShm.xdc
 *
 *  @brief      RingIO shared memory based physical transport for
 *              communication with the remote processor.
 *
 *              This file contains the declarations of types and APIs as part
 *              of interface of the RingIO shared memory transport.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


/*
 *  ======== IRingIOTransport.xdc ========
 *
 */

package ti.syslink.ipc.rtos.interfaces;


import xdc.runtime.Error;
import ti.sdo.ipc.GateMP;

/*!
 *  ======== IRingIOTransportShm ========
 *  Interface for the transports used by RingIO
 *
 *  The transport implementations have to register with
 *  {@link ti.sdo.ipc.RingIO}. This is done via the
 *  {@link ti.sdo.ipc.RingIO#registerTransport} function.
 *
 *  If transports need additional processing during startup,
 *  there are multiple hook points to run start-up code that
 *  the transport implementation can use.
 */

interface IRingIOTransportShm {

    /*!
     *  Transport status values
     *
     *  The status of the transport instance.
     *  @p(blist)
     *  -{@link #Status_INIT}: Has not not completed initialization.
     *  -{@link #Status_UP}: Transport up and functional.
     *  -{@link #Status_DOWN}: Transport down and not functional.
     *  -{@link #Status_RESETTING}: Transport was up at one point and is
     *                              in process of resetting.
     *  @p
     *
     *  All transport report {@link #Status_INIT} until they have completed
     *  initialization.
     */
    enum Status {
        Status_INIT,
        Status_UP,
        Status_DOWN,
        Status_RESETTING
    };


    /*!
     *  Module configuration structure.
     *
     *  @p(blist)
     *  - reserved
     *
     */
    struct Config {
        UInt32  reserved;
    };
    /*!
     *  ======== getConfig ========
     * Initialize the passed pointer with the current parameters
     *
     *  @param(cfgParams)   Pointer to parameter structure to be initialized.
     */
    Void  getConfig (Config * cfgParams);

    /*!
     *  ======== Setup ========
     *  Initialize the FrameQ module, there would be some default values for the
     *  parameters, but if  any change is required, change the cfgParams
     *  and call this API.
     *
     *  @param(cfgParams)   Parameters for initializing.
     */
    Int32 setup (Config * cfgParams);

    /*!
     *  ======== Destroy ========
     *  Finalize the FrameQ
     */
    Int32 destroy ();

instance:

    config UInt32  reserved = 0x0;


    /*!
     *  ======== create ========
     *  Createsa Transport instance
     *
     *  This function creates a transport instance. The transport is
     *  responsible for registering with RingIO via the
     *  {@link ti.sdo.ipc.RingIO#registerTransport} API.
     *
     *  @param(procId)        Remote processor id that this instance 
     *                        will communicate with.
     */
    create (UInt16 procId);

    /*!
     *  ======== getStatus ========
     *  Status a Transport instance
     *
     *  This function returns the status of the transport instance.
     *
     *  @b(returns)             Returns {@link #Status}
     */
    Status getStatus();

}

