/** 
 *  @file   SysLink.xdc
 *
 *  @brief      Interface file for FrameQ.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


/*
 *  ======== SysLink.xdc ========
 *
 */

import xdc.runtime.Error;
import xdc.runtime.Assert;

/*!
 *  ======== SysLink ========
 *  Common values shared by IPC modules
 *
 *  This module contains values, errors and asserts that are used by several
 *  IPC modules.
 */

module SysLink
{
    /*!
     *  Worse-case cache size
     *
     *  This is used by the IPC modules to determine the cache size.
     *  When IPC modules place things in shared memory, this value is
     *  used to make sure items are on a cache boundary and are a
     *  multiple of the size.
     *
     */
    config Int cacheSize = 128;

    /*
     *************************************************************************
     *                       Generic IPC Errors/Asserts
     *************************************************************************
     */

    /*!
     *  ======== A_invSize ========
     *  Assert raised when {@link #sharedAddrSize} is too small
     */
    config Assert.Id A_invSize  = {
        msg: "A_invSize: Invalid sharedAddrSize"
    };

    /*!
     *  ======== A_addrNotCacheAligned ========
     *  Error raised when an address is not cache-aligned
     */
    config Assert.Id A_addrNotCacheAligned  = {
        msg: "A_addrNotCacheAligned: Address is not cache aligned"
    };

    /*!
     *  ======== A_nullArgument ========
     *  Assert raised when a required argument is null
     */
    config Assert.Id A_nullArgument  = {
        msg: "A_nullArgument: Required argument is null"
    };

    /*!
     *  ======== E_versionMismatch ========
     *  Error raised in an open call because there is
     *  a version mismatch between the opener and the creator
     */
    config Error.Id E_versionMismatch  = {
        msg: "E_versionMismatch: IPC Module version mismatch: creator: %d, opener: %d"
    };

    /*
     *************************************************************************
     *                       GateSpinLock Errors/Asserts
     *************************************************************************
     */

    /*!
     *  ======== A_invSemNum ========
     *  Assert raised by GateSpinLock delegate when provided semNum is invalid
     */
    config Assert.Id A_invSemNum  = {
        msg: "A_invSemNum: Invalid hardware semaphore number"
    };

    /*
     *************************************************************************
     *                       GateSpinLock Errors/Asserts
     *************************************************************************
     */

    /*!
     *  ======== A_invSpinLockNum ========
     *  Assert raised by GateSpinLock delegate when provided lockNum is
     *  invalid
     */
    config Assert.Id A_invSpinLockNum  = {
        msg: "A_invSemNum: Invalid hardware spinlock number"
    };
    
    /*
     *************************************************************************
     *                       ListMP Errors/Asserts
     *************************************************************************
     */

    /*!
     *  ======== E_listMPNotCreated ========
     *  Error raised in {@link ti.sdo.ipc.ListMP#open} when
     *  the ListMP has not been created yet.
     */
    config Error.Id E_listMPNotCreated  = {
        msg: "E_listMPNotCreated: ListMP not created"
    };


    /*!
     *  ======== NotifyType ========
     *  Enumeration of Client notifyMgr notification types..
     *
     *  @value(NOTIFICATION_NONE)   No notification required.
     *
     *  @value(NOTIFICATION_ALWAYS) Notify whenever the other client sends data/frees up space.
     *
     *  @value(NOTIFICATION_ONCE)   Notify when the other side sends data/frees up buffer. Once the
     *                              notification is done, the notification is disabled until it is
     *                              enabled again.
     *
     *  @value(NOTIFICATION_HDWRFIFO_ALWAYS) Notify whenever the other side sends data/frees up space.
     *                                       This notification is never disabled.
     *
     *  @value(NOTIFICATION_HDWRFIFO_ONCE)   Notify when the other side sends data/frees up buffer. Once the
     *                                       notification is done, the notification is disabled until it is
     *                                       enabled again. The notification is enabled once the watermark
     *                                       is crossed and does not require buffer to get full/empty..
     */
    enum NotifyType {
		NOTIFICATION_NONE   = 0,
		NOTIFICATION_ALWAYS = 1,
		NOTIFICATION_ONCE   = 2,
		NOTIFICATION_HDWRFIFO_ALWAYS = 3,
		NOTIFICATION_HDWRFIFO_ONCE   = 4
	} ;
}

