/*
 *  @file   OsalMonitor.c
 *
 *  @brief      Linux Monitor interface implementation.
 *
 *              This abstracts the kernel side Spin Lock which is used in
 *              halting the CPU for some specified amout of time. this
 *              interface covers all aspects of Monitor required in Syslink.
 *
 *
 *  ============================================================================
 *
 *  Copyright (C) {2008-2011} Texas Instruments, Incorporated
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation version 2.
 *  
 *  This program is distributed "as is" WITHOUT ANY WARRANTY of any kind,
 *  whether express or implied; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details
 *  
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


/* Linux specific header files */
#include <linux/spinlock.h>

/* Standard headers */
#include <ti/syslink/Std.h>

/* OSAL and kernel utils */
#include <OsalMonitor.h>
#include <ti/syslink/utils/Trace.h>
#include <ti/syslink/utils/Memory.h>


#if defined (__cplusplus)
extern "C" {
#endif


/* =============================================================================
 *  Macros and types
 * =============================================================================
 */
/*!
 *  @brief   This object is used by spinlock APIs.
 */
typedef struct OsalMonitor_Object_tag {
    spinlock_t lock;
    /* !< handle to lock object used by OS.*/
    UInt32     reserved;
    /* TBD: Needed otherwise size of object comes 0. */
} OsalMonitor_Object;


/* =============================================================================
 * APIs
 * =============================================================================
 */
/*!
 *  @brief      Creates the Monitor Object.
 *
 *  @param      type    Type of spinlock (Normal or Raw)
 *
 *  @sa         OsalMonitor_delete
 */
OsalMonitor_Handle
OsalMonitor_create (OsalMonitor_Type type)
{
    OsalMonitor_Object * lockObject = NULL;

    GT_1trace (curTrace, GT_ENTER, "OsalMonitor_create", type);

    GT_assert (curTrace, (type < OsalMonitor_Type_EndValue));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (type >= OsalMonitor_Type_EndValue) {
        /*! @retval NULL Invalid type provided */
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "OsalMonitor_create",
                             OSALMONITOR_E_INVALIDARG,
                             "Invalid type provided");
    }
    else {
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */
        lockObject = Memory_alloc (NULL, sizeof (OsalMonitor_Object), 0, NULL);
#if !defined(SYSLINK_BUILD_OPTIMIZE)
        if (lockObject == NULL) {
            /*! @retval NULL Failed to allocate memory for spinlock object. */
            GT_setFailureReason (curTrace,
                              GT_4CLASS,
                              "OsalMonitor_create",
                              OSALMONITOR_E_MEMORY,
                              "Failed to allocate memory for spinlock object.");
        }
        else {
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */
            spin_lock_init(&(lockObject->lock));
#if !defined(SYSLINK_BUILD_OPTIMIZE)
        }
    }
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "OsalMonitor_create", lockObject);

    /*! @retval Monitor-handle Operation successfully completed. */
    return (OsalMonitor_Handle) lockObject;
}

/*!
 *  @brief      Delete the Monitor Object.
 *
 *  @param      lockHandle Monitor object handle which needs to be deleted.
 *
 *  @sa         OsalMonitor_create
 */
Int
OsalMonitor_delete (OsalMonitor_Handle * lockHandle)
{
    Int                   status     = OSALMONITOR_SUCCESS;
    OsalMonitor_Object * lockObject = NULL;

    GT_1trace (curTrace, GT_ENTER, "OsalMonitor_delete", lockHandle);

    GT_assert (curTrace, (lockHandle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (lockHandle == NULL) {
        /*! @retval OSALMONITOR_E_INVALIDARG NULL provided for argument
                                              lockHandle.*/
        status = OSALMONITOR_E_INVALIDARG;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "OsalMonitor_delete",
                             status,
                             "NULL provided for argument lockHandle");
    }
    else if (*lockHandle == NULL) {
        /*! @retval OSALMONITOR_E_HANDLE NULL spinlock handle provided. */
        status = OSALMONITOR_E_HANDLE;
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "OsalMonitor_delete",
                             status,
                             "NULL spinlock handle provided.");
    }
    else {
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */
        lockObject = (OsalMonitor_Object*) (*lockHandle);
        Memory_free (NULL, lockObject, sizeof (OsalMonitor_Object));
        *lockHandle = NULL;
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "OsalMonitor_delete", status);

    /*! @retval OSALMONITOR_SUCCESS Operation successfully completed. */
    return status;
}


/*!
 *  @brief      Begin protection of code through spin lock with all ISRs
 *              disabled. Calling this API protects critical regions of code
 *              from preemption by tasks, DPCs and all interrupts.
 *
 *  @param      lockHandle   Monitor object handle to be acquired.
 *
 *  @sa         OsalMonitor_leave
 */
UInt32
OsalMonitor_enter (OsalMonitor_Handle lockHandle)
{
    UInt32 flags = 0;

    OsalMonitor_Object * lockObject = (OsalMonitor_Object*) lockHandle;

    GT_1trace (curTrace, GT_ENTER, "OsalMonitor_enter", lockHandle);

    GT_assert (curTrace, (lockHandle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (lockHandle == NULL) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "OsalMonitor_enter",
                             OSALMONITOR_E_HANDLE,
                             "NULL spinlock handle provided.");
    }
    else {
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */
        spin_lock (&(lockObject->lock));
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_1trace (curTrace, GT_LEAVE, "OsalMonitor_enter", flags);

    /*! @retval Flags Flags to be provided to corresponding leave call. */
    return flags;
}


/*!
 *  @brief      End protection of code through spin lock with all ISRs disabled.
 *
 *  @param      lockHandle   Monitor object handle to be released.
 *  @param      key          Key received from the corresponding enter call.
 *
 *  @sa         OsalMonitor_enter
 */
Void
OsalMonitor_leave (OsalMonitor_Handle lockHandle, UInt32 key)
{
    OsalMonitor_Object * lockObject = (OsalMonitor_Object*) lockHandle;

    GT_2trace (curTrace, GT_ENTER, "OsalMonitor_leave", lockHandle, key);

    GT_assert (curTrace, (lockHandle != NULL));

#if !defined(SYSLINK_BUILD_OPTIMIZE)
    if (lockHandle == NULL) {
        GT_setFailureReason (curTrace,
                             GT_4CLASS,
                             "OsalMonitor_enter",
                             OSALMONITOR_E_HANDLE,
                             "NULL spinlock handle provided.");
    }
    else {
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */
        spin_unlock (&(lockObject->lock));
#if !defined(SYSLINK_BUILD_OPTIMIZE)
    }
#endif /* #if !defined(SYSLINK_BUILD_OPTIMIZE) */

    GT_0trace (curTrace, GT_LEAVE, "OsalMonitor_leave");
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
